//
//  CPDFViewController.m
//   Edit-Ctrl-Demo
//
//  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
//
//  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
//  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
//  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
//  This notice may not be removed from this file.
//

#import "CPDFViewController.h"

#import <ComPDFKit/ComPDFKit.h>
#import <compdfkit_tools/compdfkit_tools.h>

@interface CPDFViewController () <UIDocumentPickerDelegate,CPDFToolsViewControllerDelegate,CPDFEditToolBarDelegate>

@property(nonatomic, strong) CPDFEditViewController *baseVC;

@property(nonatomic, assign) CPDFEditMode editMode;

@property(nonatomic, strong) CPDFEditToolBar * toolBar;
@end

@implementation CPDFViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    CPDFEditingConfig *editingConfig = [[CPDFEditingConfig alloc]init];
    editingConfig.editingBorderWidth = 1.0;
    editingConfig.editingOffsetGap = 5;
    self.pdfListView.editingConfig = editingConfig;

    self.pdfListView.toolModel = CToolModelEdit;
    
    [self initWithEditTool];
    
    [self enterEditMode];
    [self.toolBar updateButtonState];
    
    [self enterEditMode];
}

- (void)initWithEditTool {
    if(!self.toolBar){
        self.toolBar = [[CPDFEditToolBar alloc] initWithPDFView:self.pdfListView];
    }
    
    self.toolBar.delegate = self;
    [self.view addSubview:self.toolBar];
}

- (void)initWitNavigationTitle {
    //titleButton
    CNavigationBarTitleButton * navTitleButton = [[CNavigationBarTitleButton alloc] init];
    self.titleButton = navTitleButton;
    self.navigationTitle = NSLocalizedString(@"Edit", nil);
    [navTitleButton setImage:[UIImage imageNamed:@"syasarrow"] forState:UIControlStateNormal];
    [navTitleButton addTarget:self action:@selector(titleButtonClickd:) forControlEvents:UIControlEventTouchUpInside];
    [navTitleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    [navTitleButton setTitleColor:[CPDFColorUtils CAnyReverseBackgooundColor] forState:UIControlStateNormal];
    self.titleButton.frame = CGRectMake(0, 0, 50, 30);
    self.navigationItem.titleView = self.titleButton;
}

#pragma mark - Action

- (void)titleButtonClickd:(UIButton *) button {
    CPDFToolsViewController * toolsVc = [[CPDFToolsViewController alloc] initCustomizeWithToolArrays:@[@(CToolModelViewer),@(CToolModelEdit)]];
    toolsVc.delegate = self;
    AAPLCustomPresentationController *presentationController NS_VALID_UNTIL_END_OF_SCOPE;
    presentationController = [[AAPLCustomPresentationController alloc] initWithPresentedViewController:toolsVc presentingViewController:self];
    toolsVc.transitioningDelegate = presentationController;
    [self presentViewController:toolsVc animated:YES completion:nil];
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    
    CGFloat tPosY = 0;
    CGFloat bottomY = 0;
    CGFloat height = 44.0;
    
    if (@available(iOS 11.0, *))
        height += self.view.safeAreaInsets.bottom;
  
    if(CToolModelViewer == self.pdfListView.toolModel) {
        height = 0;
    }
    
    if (!self.navigationController.navigationBarHidden) {
        CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
        tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;
        self.toolBar.frame = CGRectMake(0, self.view.frame.size.height - height, self.view.frame.size.width, height);
        bottomY = self.toolBar.frame.size.height;
    } else {
        self.toolBar.frame = CGRectMake(0, self.view.frame.size.height, self.view.frame.size.width, height);
        bottomY = 0;
    }
    
    if (@available(iOS 11.0, *)) {
        self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - tPosY- bottomY);
    } else {
        self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height - tPosY-bottomY);
    }
}

#pragma mark - Private

- (void)enterEditMode {
    self.pdfListView.toolModel = CToolModelEdit;
    self.toolBar.hidden = NO;
    self.navigationTitle = NSLocalizedString(@"Content Edit", nil);
    [self.titleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    [self.pdfListView beginEditingLoadType:CEditingLoadTypeText | CEditingLoadTypeImage];
    [self viewWillLayoutSubviews];
}

- (void)enterViewerMode {
    self.pdfListView.toolModel = CToolModelViewer;
    self.navigationTitle = NSLocalizedString(@"Viewer", nil);
    [self.titleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    if (self.pdfListView.isEdited) {
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            [self.pdfListView commitEditing];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.pdfListView endOfEditing];
            });
        });
    } else {
        [self.pdfListView endOfEditing];
    }
    
    [self viewWillLayoutSubviews];
}

- (void)enterPDFShare  {
    if(self.pdfListView.isEditing && self.pdfListView.isEdited) {
        [self.pdfListView commitEditing];
        [self.pdfListView endOfEditing];
    }
    
    [super enterPDFShare];
}

- (void)enterPDFAddFile  {
    if(self.pdfListView.isEditing && self.pdfListView.isEdited) {
        [self.pdfListView commitEditing];
        [self.pdfListView endOfEditing];
    }
    [super enterPDFAddFile];
}

#pragma mark - CPDFViewDelegate

- (void)PDFListViewCustomMenuClick:(CPDFListView *)pdfView identifier:(NSString *)menuIdentifier {
    if([menuIdentifier isEqualToString:@"PropertyItem"]) {
        self.editMode = CPDFEditModeText;
        [self showMenuList];
        [self.toolBar updateButtonState];
    }
}

- (void)PDFViewCurrentPageDidChanged:(CPDFView *)pdfView {
    [self.toolBar updateButtonState];
}


#pragma mark - CPDFToolsViewControllerDelegate

- (void)CPDFToolsViewControllerDismiss:(CPDFToolsViewController *)viewController selectItemAtIndex:(CToolModel)selectIndex {
    if(CToolModelViewer == selectIndex) {
        //viewwer
        [self enterViewerMode];
    }else{
        //edit
        [self enterEditMode];
    }
}

#pragma mark - CPDFEditToolBarDelegate

- (void)editClickInToolBar:(CPDFEditToolBar *)toolBar editMode:(CPDFEditMode)mode{
    self.editMode = mode;
}

- (void)undoDidClickInToolBar:(CPDFEditToolBar *)toolBar{
    [self.pdfListView editTextUndo];
}

- (void)redoDidClickInToolBar:(CPDFEditToolBar *)toolBar{
    [self.pdfListView editTextRedo];
}

- (void)propertyEditDidClickInToolBar:(CPDFEditToolBar *)toolBar{
    [self showMenuList];
}

- (void)showMenuList {
    _baseVC = [[CPDFEditViewController alloc] initWithPDFView:self.pdfListView];
    _baseVC.editMode = self.editMode;
    if((self.editMode == CPDFEditModeText || self.editMode == CPDFEditModeImage) && self.pdfListView.editStatus != CEditingSelectStateEmpty){
        
        AAPLCustomPresentationController *presentationController NS_VALID_UNTIL_END_OF_SCOPE;
       
        presentationController = [[AAPLCustomPresentationController alloc] initWithPresentedViewController:self.baseVC presentingViewController:self];
        self.baseVC.transitioningDelegate = presentationController;
        
        [self presentViewController:self.baseVC animated:YES completion:nil];
    }
}

@end
